unit timer_unit;

{
  microseconds clock 
}

interface

uses windows,classes,sysutils;

type  Tdav7Timer = class(TComponent)
      private
       FCPUclock : double;
       Fstart    : Int64;
       Fend      : Int64;
      protected
       function getCPUclockString : string;
       function getCPUtime : Int64;
       function getCPUTimeString  : string;
       function getElapsedTime : double;
       function getElapsedTimeString : string;
      public
       constructor create(Aowner:TComponent);  override;
       destructor destroy; override;
       property CPUclock : double read FCPUclock;
       property CPUclockString : string read getCPUclockString;
       property CPUTimeString  : string read getCPUTimeString;
       procedure start;
       procedure stop;
       property ElapsedTime : double read getElapsedTime;
       property ElapsedTimeString : string read getElapsedTimeString;
       property CPUtime : Int64 read getCPUtime;
       function CPUclockToTime(I64 : Int64) : double;
      end;


implementation


function TDav7Timer.getCPUtime : Int64;
asm
 RDTSC;  //dw $310F
end;

constructor TDav7Timer.create(Aowner : TComponent);
var t1,t2 : dword;
begin
 inherited create(AOwner);
 t2 := getTickCount;
 repeat
  t1 := getTickCount;
 until t2 <> t1;           //wait for millisecs counter to change
 Fstart := CPUtime;
 repeat
  t2 := getTickCount;
 until t2 - t1 >= 500;
 Fend := CPUtime;
 FCPUclock := (Fend - Fstart)/(t2 - t1) * 1e-3; //microseconds clock
end;

destructor TDav7Timer.destroy;
begin
 inherited destroy;
end;

procedure TDav7Timer.Start;
begin
 FStart := CPUtime;
end;

procedure TDav7Timer.Stop;
begin
 Fend := CPUtime;
end;

function TDav7Timer.getElapsedTime : double;
begin
 result := (Fend - Fstart) / FCPUclock;
end;

function TDav7Timer.CPUclockToTime(i64 : Int64) : double;
begin
 result := i64 / FCPUclock;
end;

function TDav7Timer.getElapsedTimeString : string;
const Fconvert = '#####0.0###';
var t : double;
begin
 t := getElapsedTime;
 result := formatfloat(Fconvert,t);
end;

function TDav7Timer.getCPUclockString : string;
const Fconvert = '###0.0###';
begin
 result := formatfloat(FConvert,FCPUclock);
end;

function TDav7Timer.getCPUTimeString : string;
var i64 : int64;
begin
 i64 := CPUTime;
 result := inttostr(i64);
end;

end.
